# This file is part of SSHare.
#
# SSHare is free software: you can redistribute it and/or modify it under the terms of
# the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# SSHare. If not, see <https://www.gnu.org/licenses/>.

import argparse
import getpass
import importlib.util
import os
import os.path
import pkg_resources
import pyclip
import time
import subprocess
import sys
import _version

def main():
    arguments = parse_arguments()

    config = Config()

    contents = b''
    target_file_extension = ""
    if arguments.latest or arguments.file != None:
        file_path = ""
        if arguments.latest:
            file_path = _latest(config.source_directory)
        else:
            file_path = arguments.file
        print(f"=> Uploading file '{file_path}'")

        with open(file_path, mode="rb") as file:
            contents = file.read()

        (_, target_file_extension) = os.path.splitext(file_path)
    elif arguments.paste:
        print("=> Uploading contents of clipboard")
        contents = pyclip.paste()
        target_file_extension = ".txt"
    else:
        print("=> Error: must specify one of -f FILE, -l, -p")
        sys.exit(1)
       
    target_id = time.time_ns()
    target_file_name = f"{target_id}{target_file_extension}"
    target_file = f"{config.ssh_dir}/{target_file_name}"
    target_destination = f"{config.ssh_user}@{config.ssh_target}"
    print(f"=> Uploading to '{target_destination}:{target_file}'")
    process = subprocess.run([
            "ssh",
            f"-p {config.ssh_port}",
            target_destination,
            "-T",
            f"cat - > {target_file}"
        ],
        input = contents,
    )
    if process.returncode != 0:
        print("=> Error: failed to upload file")
        sys.exit(1)

    target_url = f"https://{config.ssh_target}/sshare/{target_file_name}"
    print(f"=> File available at '{target_url}'")
    if arguments.copy:
        pyclip.copy(target_url)
        print("=> URL copied to clipboard")

    sys.exit(0)


def parse_arguments():
    parser = argparse.ArgumentParser(
        prog = "SSHare",
        description = "Upload files to a server via ssh",
    )

    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"%(prog)s version {_version.version}",
    )
    parser.add_argument(
        "-l",
        "--latest",
        action="store_const",
        const=True,
        help="Upload the latest image from the source directory",
    )
    parser.add_argument(
        "-p",
        "--paste",
        action="store_const",
        const=True,
        help="Upload the contents of the clipboard as a txt",
    )
    parser.add_argument(
        "-f",
        "--file",
        help="Upload a file",
    )
    parser.add_argument(
        "-c",
        "--copy",
        action="store_const",
        const=True,
        help="Copy the resultant url to the clipboard",
    )
    arguments = parser.parse_args()

    return arguments


def _latest(directory, key=os.path.getmtime):
    files         = map(lambda file: f"{directory}/{file}", os.listdir(directory))
    selection     = next(files)
    selection_key = key(selection)
    for file in files:
        new_key = key(file)
        if new_key > selection_key:
            selection     = file
            selection_key = key
    return selection
    

class Config:
    def __init__(self):
        self.source_directory = "/home/gnarwhal/Pictures/Screenshots"
        self.ssh_port         = 2222
        self.ssh_user         = getpass.getuser()
        self.ssh_target       = "monodon.me"
        self.ssh_dir          = "/srv/gnarwhal/sshare"
